//
// Created by xiangning zhu on 2021/12/20.
//

#ifndef SAMI_CORE_SAMI_CORE_BASE_DEF_H
#define SAMI_CORE_SAMI_CORE_BASE_DEF_H

typedef enum dataType {
    // base
    SAMICoreDataType_Float = 0,
    SAMICoreDataType_Int,
    SAMICoreDataType_String,

    // custom base
    SAMICoreDataType_AudioBuffer = 50,
    SAMICoreDataType_ExtendAudioBuffer = 51,

    // context
    SAMICoreDataType_ContextPrepareParameter = 100,
    SAMICoreDataType_ResourceParameter,
    SAMICoreDataType_ParameterEvent,

    // processor
    SAMICoreDataType_ProcessorParameter = 150,

    // Extractor
    SAMICoreDataType_FeatureSet = 250,
    SAMICoreDataType_ExtractorParameter,

    // model binary
    SAMICoreDataType_ModelBin = 300,

    // aec
    SAMICoreDataType_AecInput = 350,

    // sami engine tts
    SAMICoreDataType_TTSCallBack = 400,
    SAMICoreDataType_EngineCreateParameter,
    SAMICoreDataType_ExecutorCreateParameter,

    // sami engine wakeup
    SAMICoreDataType_Engine_WakeupWord_Result = 410,

    // time align
    SAMICoreDataType_TimeAlign = 450,

    //loud norm
    SAMICoreDataType_LoudNorm = 451,

    //loud norm
    SAMICoreDataType_LoudNorm2 = 452,

    // audio fading
    SAMICoreDataType_AudioFading = 453,

    // cher effect
    SAMICoreDataType_CherEffect = 454,

    // kws
    SAMICoreDataType_KWS = 455,

    // AudioEffect Compressor
    SAMICoreDataType_AudioEffect_Compressor = 456,

    // server
    SAMICoreDataType_TTS_Result = 500,
    SAMICoreDataType_ASR_Result = 503,
    SAMICoreDataType_AudioBin = 504,
    SAMICoreDataType_BeatTracking_Result = 505,
    SAMICoreDataType_MusicSourceSeparate_Result = 506,
    SAMICoreDataType_MusicTagging_Result = 507,
    SAMICoreDataType_Chorus_Result = 508,
    SAMICoreDataType_MusicRetarget_Result = 509,
    SAMICoreDataType_Midi_Result = 510,
    SAMICoreDataType_Token_Result = 511,
    SAMICoreDataType_SpeechDisfluency_Result = 512,
    SAMICoreDataType_VoiceFilterParam = 513,
    SAMICoreDataType_ServerHttpResponse = 514,
    SAMICoreDataType_LyricsAlignment_Result = 515,

    // websocket
    SAMICoreDataType_WebSocket_Server_Event = 600,
    SAMICoreDataType_WebSocket_Connection_Event = 601,
    SAMICoreDataType_WebSocket_Asr = 602,

    // yggdrasil
    SAMICoreDataType_Yggdrasil_Mesh = 700,
    SAMICoreDataType_Yggdrasil_Mesh_And_Material_Factor,
    SAMICoreDataType_Yggdrasil_Factor,
    SAMICoreDataType_Yggdrasil_Source,
    SAMICoreDataType_Yggdrasil_Source_With_Orientation ,
    SAMICoreDataType_Yggdrasil_Source_Buffer ,
    SAMICoreDataType_Yggdrasil_Ambisonic_Channel_Buffer ,
    SAMICoreDataType_Yggdrasil_Matrix_Input_Buffer ,
    SAMICoreDataType_Yggdrasil_Binaural_Buffer ,
    SAMICoreDataType_Yggdrasil_Loudspeaker_Buffer,
    SAMICoreDataType_Yggdrasil_Doppler_Effect,
    SAMICoreDataType_Yggdrasil_Loudspeaker_Array,
    SAMICoreDataType_Yggdrasil_Mapping_Matrix,
    SAMICoreDataType_Yggdrasil_Ambisonic_Orientation,
    SAMICoreDataType_Yggdrasil_Listener,

    // MDSS
    SAMICoreDataType_MDSS_RealtimeInfo,
    SAMICoreDataType_MDSS_OverallInfo,
} SAMICoreDataType;

typedef enum identify {
    /**
     * SAMICoreIdentify_Processor_ContextWithString, used to create vocal preset from json string.
     * create handle with SAMICoreContextCreateParameter
     *
     * @see SAMICoreContextCreateParameter
     */
    SAMICoreIdentify_Processor_ContextWithString = 0,

    /**
     * SAMICoreIdentify_Processor_ContextWithFileName, used to create vocal preset from json file path.
     * create handle with SAMICoreContextCreateParameter
     *
     * @see SAMICoreContextCreateParameter
     */
    SAMICoreIdentify_Processor_ContextWithFileName,

    /**
     * SAMICoreIdentify_Processor, used to create processor by name,here is an example:
     * @code
     *  SAMICoreHandle handle;
     *  const char* name = "Gain";
     *  int ret = SAMICoreCreateHandleByIdentify(&handle, SAMICoreIdentify_Processor, name);
     *  assert(ret == SAMI_OK);
     * @endcode
     *
     * or, you can create processor by enum id, like this:
     * @code
     * int ret = SAMICoreCreateHandleByIdentify(&handle, SAMICoreIdentify_Processor_Compressor, NULL);
     * @endcode
     *
     * the currently supported processors' name are match to processor id,
     * here is a table shows the mapping between IDs and names
     *
     * +---------------------------------------+--------------+
     * | Id                                    | Name         |
     * +---------------------------------------+--------------+
     * | SAMICoreIdentify_Processor_Gain       | "Gain"       |
     * +---------------------------------------+--------------+
     * | SAMICoreIdentify_Processor_Compressor | "Compressor" |
     * +---------------------------------------+--------------+
     *
     *
     */
    SAMICoreIdentify_Processor = 100,

    /**
     * Name: "Gain"
     * Parameters:
     * +---------+-------------+-----------+-------------+------------+
     * | index   | name        | type      | range       | default    |
     * +---------+-------------+-----------+-------------+------------+
     * | 0       | "Gain dB"   | "Float"   | [-70, 35]   | 0          |
     * +---------+-------------+-----------+-------------+------------+
     */
    SAMICoreIdentify_Processor_Gain,

    /**
     * Name: "Compressor"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     *  | index   | name                 | type      | range           | default    |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 0       | "Bypass"             | "Bool"    | [false, true]   | false      |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 1       | "Ratio"              | "Float"   | [1, 40]         | 1          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 2       | "Threshold dB"       | "Float"   | [-70, 0]        | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 3       | "Knee dB"            | "Float"   | [0, 18]         | 6          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 4       | "Attack"             | "Float"   | [0.005, 250]    | 0.25       |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 5       | "Release"            | "Float"   | [1, 2500]       | 100        |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 6       | "Auto Makeup Gain"   | "Bool"    | [false, true]   | true       |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 7       | "Output Gain dB"     | "Float"   | [-35, 35]       | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 8       | "Lookahead"          | "Float"   | [0, 10]         | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Compressor,

    /**
     * Name: "Chorus"
     * Parameters:
     * +---------+-------------------------+-----------+-----------------+------------+
     * | index   | name                    | type      | range           | default    |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"                | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 1       | "Delay"                 | "Float"   | [0.5, 20]       | 4          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 2       | "Rate"                  | "Float"   | [0, 4]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 3       | "Depth"                 | "Float"   | [0, 5]          | 2          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 4       | "Stereo Phase Offset"   | "Float"   | [0, 1]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 5       | "Feedback"              | "Float"   | [0, 0.96]       | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 6       | "Wetness"               | "Float"   | [0, 1]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Chorus,

    /**
     * Name: "Distortion"
     * Parameters:
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | index   | name                     | type       | range | default    |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 0       | "Bypass"                 | "Bool"     | [false, true] | false      |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 1       | "Distortion Mode"        | "Choice"   | ["Cubic", "Tanh", "Arctan", "ArctanTanh", "L1", "L2",
     * "HardClip", "DeadZone", "FoldBackTriangle", "FoldBackSine", "FoldBackSinArctan", "BitCrush", "Decimate"]   |
     * "Cubic"    |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 2       | "Distortion Amount"      | "Float"    | [0, 1] | 0          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 3       | "Apply Upsampling"       | "Choice"   | ["None", "X8", "X16", "X32"] | "None"     |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 4       | "Wetness"                | "Float"    | [0, 1] | 1          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 5       | "Post Distortion Gain"   | "Float"    | [-40, 0] | 0          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     */
    SAMICoreIdentify_Processor_Distortion,

    /**
     * Name: "Echo"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     * | index   | name                 | type      | range           | default    |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 0       | "Dry Bypass"         | "Bool"    | [false, true]   | false      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 1       | "Dry Gain dB"        | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 2       | "Dry Pan"            | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 3       | "Tap 1 Bypass"       | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 4       | "Tap 1 Delay Time"   | "Float"   | [0.001, 1]      | 0.001      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 5       | "Tap 1 Feedback"     | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 6       | "Tap 1 Gain dB"      | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 7       | "Tap 1 Pan"          | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 8       | "Tap 1 Sync"         | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 9       | "Tap 2 Bypass"       | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 10      | "Tap 2 Delay Time"   | "Float"   | [0.001, 1]      | 0.001      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 11      | "Tap 2 Feedback"     | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 12      | "Tap 2 Gain dB"      | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 13      | "Tap 2 Pan"          | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 14      | "Tap 2 Sync"         | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Echo,

    /**
     * Name: "Filter"
     * Parameters:
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | index   | name            | type       | range | default      |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 0       | "Filter Mode"   | "Choice"   | ["LowPass", "HighPass", "BandPass", "NormalisedBandPass", "Bell",
     * "HighShelf", "LowShelf", "AllPass", "Notch", "Morph"]   | "LowPass"    |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 1       | "Cutoff Freq"   | "Float"    | [20, 20000] | 1000         |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 2       | "Resonance"     | "Float"    | [0.1, 20] | 1            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 3       | "Gain dB"       | "Float"    | [-20, 20] | 0            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 4       | "Morph"         | "Float"    | [-1, 1] | 0            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     */
    SAMICoreIdentify_Processor_Filter,

    /**
     * Name: "GainAndPan"
     * Parameters:
     * +---------+-------------+-----------+-------------+------------+
     * | index   | name        | type      | range       | default    |
     * +---------+-------------+-----------+-------------+------------+
     * | 0       | "Gain dB"   | "Float"   | [-70, 35]   | 0          |
     * +---------+-------------+-----------+-------------+------------+
     * | 1       | "Pan"       | "Float"   | [-1, 1]     | 0          |
     * +---------+-------------+-----------+-------------+------------+
     */
    SAMICoreIdentify_Processor_GainAndPan,

    /**
     * Name: "Limiter"
     * Parameters:
     * +---------+-------------------------+-----------+-----------------+------------+
     * | index   | name                    | type      | range           | default    |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"                | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 1       | "Input Gain dB"         | "Float"   | [0, 24]         | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 2       | "Ceiling dB"            | "Float"   | [-24, 0]        | -0.3       |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 3       | "Release"               | "Float"   | [1, 3000]       | 500        |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 4       | "Lookahead"             | "Float"   | [0, 10]         | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 5       | "Audition Unity Gain"   | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Limiter,

    /**
     * Name: "Reverb"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     * | index   | name                 | type      | range           | default    |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"             | "Bool"    | [false, true]   | false      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 1       | "Decay Time"         | "Float"   | [0.5, 16]       | 3          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 2       | "Damp"               | "Float"   | [0, 1]          | 0.5        |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 3       | "Mod Depth"          | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 4       | "Mod Rate"           | "Float"   | [0.01, 7.5]     | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 5       | "Wet Gain"           | "Float"   | [-36, 12]       | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 6       | "Wet Stereo Width"   | "Float"   | [0, 2]          | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 7       | "Wet Mix"            | "Float"   | [0, 1]          | 0.5        |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 8       | "Wet LP Bypass"      | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 9       | "Wet HP Bypass"      | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 10      | "Wet LP Freq"        | "Float"   | [20, 20000]     | 20000      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 11      | "Wet HP Freq"        | "Float"   | [20, 20000]     | 20         |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 12      | "Wet LP Res"         | "Float"   | [0.1, 20]       | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 13      | "Wet HP Res"         | "Float"   | [0.1, 20]       | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Reverb,

    /**
     * Name: "Vibrato"
     * Parameters:
     * +---------+--------------------+-----------+-----------------+------------+
     * | index   | name               | type      | range           | default    |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"           | "Bool"    | [false, true]   | false      |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 1       | "Rate Hz"          | "Float"   | [1, 8]          | 4          |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 2       | "Depth Semitone"   | "Float"   | [0, 2]          | 0.5        |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 3       | "Wetness"          | "Float"   | [0, 1]          | 1          |
     * +---------+--------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Vibrato,

    /**
     * Name: "TimeDomainPitchShifter"
     * Parameters:
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | index   | name                   | type       | range                                                                        | default        |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 0       | "Bypass"               | "Bool"     | [false, true]                                                                | false          |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 1       | "Pitch Ratio"          | "Float"    | [0.25, 4]                                                                    | 1              |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 2       | "Interpolation Mode"   | "Choice"   | ["Linear", "Lagrange4", "Lagrange16", "Lagrange24", "Sinc32", "Lanczos32"]   | "Lagrange4"    |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 3       | "Mono Input"           | "Bool"     | [false, true]                                                                | true           |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 4       | "Mode"                 | "Choice"   | ["Fastest", "Moderate", "HighQuality", "UltraHighQuality"]                   | "Moderate"     |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     */
    SAMICoreIdentify_Processor_TimeDomainPitchShifter,

    /**
     * Name: "SpatialAudio"
     * Parameters:
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | index   | name                   | type       | range                                                                        | default        |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 0       | "preset_id"            | "Int"      | [0, 1]                                                                       | 1              |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     */
    SAMICoreIdentify_Processor_SpatialAudio,

    /**
     * Name: "AECompressor"
     * Parameters:
     * +-------+--------------------------+---------+---------------+---------+
       | index | name                     | type    | range         | default |
       +-------+--------------------------+---------+---------------+---------+
       | 0     | Bypass                   | "Bool"  | [false, true] | false   |
       +-------+--------------------------+---------+---------------+---------+
       | 1     | Pre Gain dB              | "Float" | [0.0, 100]    | 0       |
       +-------+--------------------------+---------+---------------+---------+
       | 2     | Threshold                | "Float" | [-100, 0]     | -24     |
       +-------+--------------------------+---------+---------------+---------+
       | 3     | Knee                     | "Float" | [0.0, 40]     | 30      |
       +-------+--------------------------+---------+---------------+---------+
       | 4     | Ratio                    | "Float" | [0.0, 20]     | 12      |
       +-------+--------------------------+---------+---------------+---------+
       | 5     | Attack                   | "Float" | [0.0, 1]      | 0.003   |
       +-------+--------------------------+---------+---------------+---------+
       | 6     | Release                  | "Float" | [0.0, 1]      | 0.25    |
       +-------+--------------------------+---------+---------------+---------+
       | 7     | Pre Delay                | "Float" | [0.0, 1]      | 0.006   |
       +-------+--------------------------+---------+---------------+---------+
       | 8     | Release Zone 1           | "Float" | [0.0, 1]      | 0.09    |
       +-------+--------------------------+---------+---------------+---------+
       | 9     | Release Zone 2           | "Float" | [0.0, 1]      | 0.16    |
       +-------+--------------------------+---------+---------------+---------+
       | 10    | Release Zone 3           | "Float" | [0.0, 1]      | 0.42    |
       +-------+--------------------------+---------+---------------+---------+
       | 11    | Release Zone 4           | "Float" | [0.0, 1]      | 0.98    |
       +-------+--------------------------+---------+---------------+---------+
       | 12    | Post Gain dB             | "Float" | [0.0, 100]    | 0.0     |
       +-------+--------------------------+---------+---------------+---------+
       | 13    | Wet                      | "Float" | [0.0, 1]      | 1.0     |
       +-------+--------------------------+---------+---------------+---------+
       | 14    | Attenuation dB Threshold | "Float" | [0.0, 2]      | 2.0     |
       +-------+--------------------------+---------+---------------+---------+
       | 15    | Detector Avg Threshold   | "Float" | [0.0, 1]      | 1.0     |
       +-------+--------------------------+---------+---------------+---------+
     */
    SAMICoreIdentify_Processor_AECompressor,

    // extractor
    SAMICoreIdentify_Extractor_F0Detection = 200,
    SAMICoreIdentify_Extractor_OnsetDetection,
    SAMICoreIdentify_Extractor_SpectrumDisplay,
    SAMICoreIdentify_Extractor_VolumeDetection,
    SAMICoreIdentify_Extractor_EventDetection,
    SAMICoreIdentify_Extractor_VoiceActivityDetection,
    SAMICoreIdentify_Extractor_BeatTracking,
    SAMICoreIdentify_Extractor_BeatTrackingOffline,
    SAMICoreIdentify_Extractor_Music2VibesVideoModel,
    SAMICoreIdentify_Extractor_Music2VibesMatchModel,
    SAMICoreIdentify_Extractor_NNVAD,
    SAMICoreIdentify_Extractor_Loudness,

    // 3a
    SAMICoreIdentify_AEC = 300,
    SAMICoreIdentify_RNNAEC,
    SAMICoreIdentify_AGC,
    SAMICoreIdentify_RNNOSIE16K,
    SAMICoreIdentify_RNNOSIE,
    SAMICoreIdentify_RNNOSIE48K,
    SAMICoreIdentify_TCNDENOISE44K,
    SAMICoreIdentify_TCNDENOISE16K,
    SAMICoreIdentify_DENOISE_V2_MUSIC44k,
    SAMICoreIdentify_DENOISE_V2_SPEECH44k,
    SAMICoreIdentify_DENOISE_V2_MUSIC16k,
    SAMICoreIdentify_DENOISE_V2_SPEECH16k,
    SAMICoreIdentify_DENOISE_V2_DEREVERB44k,
    SAMICoreIdentify_BEAM_FORMING,
    SAMICoreIdentify_NNAEC_MIC_SELECTION,
    SAMICoreIdentify_DENOISE_V2,
    SAMICoreIdentify_Loudnorm,
    SAMICoreIdentify_Loudnorm2,
    SAMICoreIdentify_AudioFading,
    SAMICoreIdentify_CherEffect,
    SAMICoreIdentify_AudioEffect_Compressor,
    SAMICoreIdentify_RNNAEC_V3,
    // non realtime effect
    SAMICoreIdentify_TimeScaler = 400,
    SAMICoreIdentify_PitchShifter = 401,

    // sami engine
    SAMICoreIdentify_Engine = 500,
    SAMICoreIdentify_EngineExecutor,
    SAMICoreIdentify_EngineExecutor_TimeAlign,
    SAMICoreIdentify_EngineExecutor_AED,
    SAMICoreIdentify_EngineExecutor_Denoise,
    SAMICoreIdentify_EngineExecutor_WakeupWord,
    SAMICoreIdentify_EngineExecutor_MSS,
    SAMICoreIdentify_EngineExecutor_KWS,
    SAMICoreIdentify_EngineExecutor_AudioMetrics,

    // server
    SAMICoreIdentify_Create_Token = 600,
    SAMICoreIdentify_TTS_Online = 601,
    SAMICoreIdentify_ASR_Online = 602,
    SAMICoreIdentify_BeatTracking_Online = 603,
    SAMICoreIdentify_MusicSourceSeparate_Online = 604,
    SAMICoreIdentify_MusicTagging_Online = 605,
    SAMICoreIdentify_Chorus_Online = 606,
    SAMICoreIdentify_MusicRetarget_Online = 607,
    SAMICoreIdentify_Midi_Online = 608,
    SAMICoreIdentify_SpeechDisfluency_Online = 609,
    SAMICoreIdentify_VoiceFilter_Online = 610,
    SAMICoreIdentify_GET_MixedEncryptedFOnlineToken = 611,
    SAMICoreIdentify_LyricsAlignment_Online = 612,


    SAMICoreIdentify_Streaming_ASR_Online = 650,
    SAMICoreIdentify_Streaming_TTS_Online = 651,

    // asr
    SAMICoreIdentify_SPEECH_RECOGNITION = 700,
    SAMICoreIdentify_ASR_Offline = 701,

    // audio encode
    SAMICoreIdentify_Audio_Encode = 800,

    // non-effect
    SAMICoreIdentify_AudioScratcher = 900,

    //tob
    SAMICoreIdentify_Processor_ContextWithBinaryRes = 2000,

    SAMICoreIdentify_TuneToTarget = 3000,

    // yggdrasil
    SAMICoreIdentify_Yggdrasil = 4000,

    // audio_norm
    SAMICoreIdentify_AudioNorm = 5000,

    // MDSS
    SAMICoreIdentify_MulDimSingScoring = 6000,   
    
} SAMICoreIdentify;

typedef enum propertyId {
    // base parameter
    SAMICorePropertyID_Base_NAME = 0,
    SAMICorePropertyID_Base_UNKNOWN,

    // context
    SAMICorePropertyID_Processor_ContextPrepare = 100,
    SAMICorePropertyID_JSON_Context,

    // vad parameters
    SAMICorePropertyID_VAD_Aggressiveness_Mode_Name = 200,
    SAMICorePropertyID_VAD_Silence_Robust_Length_name,
    SAMICorePropertyID_VAD_Frame_Time_Length_Name,
    SAMICorePropertyID_VAD_Smoothing_Searching_Width_Name,
    SAMICorePropertyID_VAD_Smoothing_Searching_Threshold_Ratio_Name,
    SAMICorePropertyID_VAD_Overall_Moving_Average_Width_Name,
    SAMICorePropertyID_VAD_Overall_Searching_Threshold_Ratio_Name,
    SAMICorePropertyID_VAD_Overall_Silence_Length_Name,

    // compressor
    SAMICorePropertyID_Compressor_Bypass = 300,
    SAMICorePropertyID_Compressor_Ratio,
    SAMICorePropertyID_Compressor_Threshold,
    SAMICorePropertyID_Compressor_Knee_dB,
    SAMICorePropertyID_Compressor_Attack,
    SAMICorePropertyID_Compressor_Release,
    SAMICorePropertyID_Compressor_Auto_Makeup_Gain,
    SAMICorePropertyID_Compressor_Output_Gain_dB,
    SAMICorePropertyID_Compressor_Look_Head,

    // extractor frame feature
    SAMICorePropertyID_FrameFeatures = 400,
    SAMICorePropertyID_FrameFeature_SPECTRUM,
    SAMICorePropertyID_FrameFeature_ONSET,
    SAMICorePropertyID_FrameFeature_F0,
    SAMICorePropertyID_FrameFeature_BEAT_TRACKING,
    SAMICorePropertyID_FrameFeature_VOLUME,
    SAMICorePropertyID_FrameFeature_NNVAD,
    SAMICorePropertyID_FrameFeature_Voice_Activity,
    SAMICorePropertyID_FrameFeature_KWS,
    SAMICorePropertyID_FrameFeature_AudioMetrics,

    // extractor overall feature
    SAMICorePropertyID_OverallFeatures = 500,
    SAMICorePropertyID_OverallFeature_GLOBAL_LOUDNESS,
    SAMICorePropertyID_OverallFeature_GLOBAL_PEAK,
    SAMICorePropertyID_OverallFeature_TRACKING_OFFLINE_OVERALL,
    SAMICorePropertyID_OverallFeature_NNVAD_OVERALL,
    SAMICorePropertyID_OverallFeature_ONSET_OVERALL,
    SAMICorePropertyID_OverallFeature_Voice_Activity_OVERALL,

    // extractor
    SAMICorePropertyID_SetExtractorFloatParameter = 600,
    SAMICorePropertyID_SetExtractorStringParameter,
    SAMICorePropertyID_GetLatency,

    // model path
    SAMICorePropertyId_LoadModelBinary = 700,

    // seek
    SAMICorePropertyId_Seek = 800,

    // time scaler
    SAMICorePropertyId_TimeScaler_ScaleRate = 900,

    // pitch shifter
    SAMICorePropertyId_PitchShifter_Value = 901,

    // audio scratcher
    SAMICorePropertyId_AudioScratcher_Value = 902,

    // processor
    SAMICorePropertyID_Processor_Prepare = 1000,
    SAMICorePropertyID_Processor_SetParametersOffline,
    SAMICorePropertyID_Processor_ContextUpdateFromFile,
    SAMICorePropertyID_Processor_ContextUpdateFromString,
    SAMICorePropertyID_Processor_ContextEmplaceParameterEventNow,  // use normalization value
    SAMICorePropertyID_Processor_ContextEmplaceParameterEventNowWithPlainValue,  // use plain value. For string options you can use the index of option as plain value
    SAMICorePropertyID_Processor_Name,
    SAMICorePropertyID_Processor_Loudnorm,
    SAMICorePropertyID_Processor_Loudnorm2,
    SAMICorePropertyID_Processor_AudioFading,
    SAMICorePropertyID_Processor_CherEffect,
    SAMICorePropertyID_Processor_AudioEffect_Compressor,

    // sami engine
    SAMICoreEngineExecutorOutPut = 1100,
    SAMICorePropertyID_KWS_KeyWords,
    SAMICorePropertyID_KWS_Language,

    // denoise
    SAMICorePropertyID_TCNDenoise = 1200,
    SAMICorePropertyID_TCNDenoise_Rate,
    SAMICorePropertyID_TCNDenoise_Mode,

    SAMICorePropertyID_Processor_ContextChunkBinaryRes = 1300,
    SAMICorePropertyID_Processor_ContextUpdateFromBinaryFile,
    SAMICorePropertyID_Processor_ContextUpdateFromBinaryRes,

    // denoise
    SAMICorePropertyID_Denoise_V2 = 1400,
    SAMICorePropertyID_Denoise_V2_Rate,
    SAMICorePropertyID_Denoise_V2_Mode,

    // speech recognition
    SAMICorePropertyID_SpeechRecognition = 1500,
    SAMICorePropertyID_SpeechRecognition_Reset,
    SAMICorePropertyID_SpeechRecognition_Finish,
    SAMICorePropertyID_SpeechRecognition_UseWordInfo,
    SAMICorePropertyID_SpeechRecognition_Resume,
    SAMICorePropertyID_SpeechRecognition_UpdateRange,

    SAMICorePropertyID_ASR_Offline,
    SAMICorePropertyID_ASR_Offline_Set_Parameters,
    SAMICorePropertyID_ASR_Offline_Rest_Executor,
    SAMICorePropertyID_ASR_Offline_Finish,
    SAMICorePropertyID_ASR_Offline_Finish_Result,
    SAMICorePropertyID_AudioMetrics_Rest_Executor,


    // yggdrasil
    SAMICorePropertyId_Yggdrasil_Initialize_Context = 1600,
    SAMICorePropertyId_Yggdrasil_Mesh,
    SAMICorePropertyId_Yggdrasil_Mesh_And_Material_Factor,
    SAMICorePropertyId_Yggdrasil_Absorption_Factor ,
    SAMICorePropertyId_Yggdrasil_Scattering_Factor ,
    SAMICorePropertyId_Yggdrasil_Transmission_Factor ,
    SAMICorePropertyId_Yggdrasil_Scene_Commit ,
    SAMICorePropertyId_Yggdrasil_Scene_Update ,
    SAMICorePropertyId_Yggdrasil_Source_Add ,
    SAMICorePropertyId_Yggdrasil_Source_With_Orientation ,
    SAMICorePropertyId_Yggdrasil_Source_Remove ,
    SAMICorePropertyId_Yggdrasil_Source_Buffer ,
    SAMICorePropertyId_Yggdrasil_Ambisonic_Channel_Buffer ,
    SAMICorePropertyId_Yggdrasil_Interleaved_Ambisonic_Buffer,
    SAMICorePropertyId_Yggdrasil_Matrix_Input_Buffer ,
    SAMICorePropertyId_Yggdrasil_Interleaved_Binaural_Buffer ,
    SAMICorePropertyId_Yggdrasil_Planar_Binaural_Buffer,
    SAMICorePropertyId_Yggdrasil_Interleaved_Loudspeaker_Buffer,
    SAMICorePropertyId_Yggdrasil_Planar_Loudspeaker_Buffer,
    SAMICorePropertyId_Yggdrasil_Doppler_Effect,
    SAMICorePropertyId_Yggdrasil_Playback_Mode,
    SAMICorePropertyId_Yggdrasil_Loudspeaker_Array,
    SAMICorePropertyId_Yggdrasil_Mapping_Matrix,
    SAMICorePropertyId_Yggdrasil_Ambisonic_Orientation,
    SAMICorePropertyId_Yggdrasil_Listener,
    SAMICorePropertyId_Yggdrasil_Source ,

    // MDSS
    SAMICorePropertyID_FrameFeature_MulDimSingScoring,
    SAMICorePropertyID_OverallFeature_MulDimSingScoring,
    SAMICorePropertyID_SentenceScores_MulDimSingScoring,
    SAMICorePropertyID_SeekTo_MulDimSingScoring,

    SAMICorePropertyID_LyricsAlignment_LyricsWords,
    SAMICorePropertyID_LyricsAlignment_AudioFileData,

    SAMICorePropertyId_ASR_Online_Reset_Recognition = 1700,
    SAMICorePropertyID_Stream_TTS_Online_Force_Finish = 1701,
    SAMICorePropertyID_Stream_TTS_Online_Wait_Stop = 1702,

    // audio metrics
    SAMICorePropertyID_AudioMetricsJson = 1800,
    SAMICorePropertyID_AudioMetrics_InputEnd,

    // nnaec v3
    SAMICorePropertyID_NNAEC_V3 = 1850,
    SAMICorePropertyID_NNAEC_V3_ExtraData,

    // add parameters of other algorithms

} SAMICorePropertyId;

struct audioBuffer {
    unsigned int numberChannels;
    unsigned int numberSamples;
    int isInterleave;
    float** data;
};

typedef struct audioBlock {
    SAMICoreDataType dataType;
    unsigned int numberAudioData;
    // abstract, can convert to any object such as midi, etc
    void* audioData;
} SAMICoreBlock;

#endif  //SAMI_CORE_SAMI_CORE_BASE_DEF_H
